@@ -2,12 +2,61 @@
 
 import os
 import configparser
+import re
 
-from Qt import QtWidgets
+import requests
+from PySide6 import QtWidgets, QtGui
 import qtawesome as qta
 from pynput.keyboard import Key, Controller
 from platformdirs import user_config_dir
 
+from NodeGraphQt6.constants import PortEnum
+
+from version import VERSION
+
+directory = os.path.dirname(__file__)
+root_dir, _ = os.path.split(directory)
+
+icon_for_type = {'BusNode.BusNode': 'ph.git-commit',
+                 'LineNode.LineNode': 'ph.line-segment',
+                 'StdLineNode.StdLineNode': 'ph.line-segment',
+                 'DCLineNode.DCLineNode': 'ph.line-segment',
+                 'ImpedanceNode.ImpedanceNode': 'mdi6.alpha-z-box-outline',
+                 'TrafoNode.TrafoNode': 'ph.intersect',
+                 'StdTrafoNode.StdTrafoNode': 'ph.intersect',
+                 'Trafo3wNode.Trafo3wNode': 'ph.intersect',
+                 'StdTrafo3wNode.StdTrafo3wNode': 'ph.intersect',
+                 'GenNode.GenNode': 'mdi6.alpha-g-circle-outline',
+                 'SGenNode.SGenNode': 'mdi6.alpha-g-circle-outline',
+                 'ASGenNode.ASGenNode': 'mdi6.alpha-g-circle-outline',
+                 'ExtGridNode.ExtGridNode': 'mdi6.grid',
+                 'LoadNode.LoadNode': 'mdi6.download-circle-outline',
+                 'ALoadNode.ALoadNode': 'mdi6.download-circle-outline',
+                 'ShuntNode.ShuntNode': 'mdi6.download-circle-outline',
+                 'MotorNode.MotorNode': 'mdi6.download-circle-outline',
+                 'WardNode.WardNode': 'mdi6.download-circle-outline',
+                 'XWardNode.XWardNode': 'mdi6.download-circle-outline',
+                 'StorageNode.StorageNode': 'mdi6.battery-medium',
+                 'SwitchNode.SwitchNode': 'mdi6.electric-switch'}
+
+
+def natsort(s):
+    """
+    Function key for natural sorting.
+
+    s: List with strings.
+    """
+    return [int(t) if t.isdigit() else t.lower() for t in re.split('(\d+)', s)]
+
+
+def natsort2(s):
+    """
+    Function key for natural sorting.
+
+    s: List obtained as enumerate(list with strings).
+    """
+    return [int(t) if t.isdigit() else t.lower() for t in re.split('(\d+)', s[1])]
+
 
 def show_WIP(window_parent):
     """
@@ -29,24 +78,52 @@ def simulate_ESC_key():
     keyboard = Controller()
     keyboard.press(Key.esc)
     keyboard.release(Key.esc)
+    
+
+def simulate_H_key():
+    """
+    Simulate the press and release of H key.
+    """
+    keyboard = Controller()
+    keyboard.press('h')
+    keyboard.release('h')
+
+
+def simulate_V_key():
+    """
+    Simulate the press and release of V key.
+    """
+    keyboard = Controller()
+    keyboard.press('v')
+    keyboard.release('v')
+
+
+def egs_path():
+    """
+    Returns the EGS installation path (directory).
+    """
+    return root_dir
 
 
 class QMainWindow2(QtWidgets.QMainWindow):
     """
     Same as default QMainWindow, but ask for permission before closing.
     """
-    def __init__(self, **kwargs):
+    def __init__(self, main_window, **kwargs):
         super().__init__(**kwargs)
+        self.main_window = main_window
+        self.resize(1399, 790)
+        self.setCentralWidget(self.main_window)
         
     def closeEvent(self, event):
-        status_msg = self.statusbar.findChild(QtWidgets.QWidget, 'unsaved_message')
+        status_msg = self.main_window.statusbar.findChild(QtWidgets.QWidget, 'unsaved_message')
         if not status_msg.isVisible():
             event.accept()
             return
         
         event.ignore()
         
-        box = QtWidgets.QMessageBox(parent=self)
+        box = QtWidgets.QMessageBox(parent=self.main_window)
         box.setIcon(QtWidgets.QMessageBox.Question)
         box.setWindowTitle('Exit the application')
         box.setText('Session is unsaved. All the data will be lost when closing the application. '+
@@ -55,7 +132,7 @@ class QMainWindow2(QtWidgets.QMainWindow):
         buttonY = box.button(QtWidgets.QMessageBox.Yes)
         buttonN = box.button(QtWidgets.QMessageBox.No)
         box.setDefaultButton(buttonN)
-        box.exec_()
+        box.exec()
         button_response = box.clickedButton()
 
         if button_response==buttonY:
@@ -72,19 +149,19 @@ class QVLine(QtWidgets.QFrame):
         self.setFrameShadow(QtWidgets.QFrame.Sunken)
 
 
-class NodeMovedCmd(QtWidgets.QUndoCommand):
+class NodeMovedCmd(QtGui.QUndoCommand):
     """
     Node moved command. (MODIFIED VERSION)
 
     Args:
-        node (NodeGraphQt.NodeObject): node.
+        node (NodeGraphQt6.NodeObject): node.
         pos (tuple(float, float)): new node position.
         prev_pos (tuple(float, float)): previous node position.
         graph: ElectricalGraph intance.
     """
 
     def __init__(self, node, pos, prev_pos, graph):
-        QtWidgets.QUndoCommand.__init__(self)
+        QtGui.QUndoCommand.__init__(self)
         self.node = node
         self.pos = pos
         self.prev_pos = prev_pos
@@ -96,7 +173,7 @@ class NodeMovedCmd(QtWidgets.QUndoCommand):
         self.node.view.xy_pos = self.prev_pos
         self.node.model.pos = self.prev_pos
         if self.node.type_=='BusNode.BusNode':
-            self.graph.net.bus_geodata.loc[self.bus_index, ['x', 'y']] = self.prev_pos
+            self.graph.net.bus.loc[self.bus_index, 'geo'] = f'{{"coordinates":[{self.prev_pos[0]: .2f},{-1*self.prev_pos[1]: .2f}], "type":"Point"}}'
 
     def redo(self):
         if self.pos == self.prev_pos:
@@ -104,7 +181,7 @@ class NodeMovedCmd(QtWidgets.QUndoCommand):
         self.node.view.xy_pos = self.pos
         self.node.model.pos = self.pos
         if self.node.type_=='BusNode.BusNode':
-            self.graph.net.bus_geodata.loc[self.bus_index, ['x', 'y']] = self.pos
+            self.graph.net.bus.loc[self.bus_index, 'geo'] = f'{{"coordinates":[{self.pos[0]: .2f},{-1*self.pos[1]: .2f}], "type":"Point"}}'
 
 
 class StatusMessageUnsaved(QtWidgets.QWidget):
@@ -127,21 +204,51 @@ class StatusMessageUnsaved(QtWidgets.QWidget):
         self.setLayout(layout)
 
 
-class PropertyChangedCmd(QtWidgets.QUndoCommand):
+class StatusFileName(QtWidgets.QWidget):
+    """
+    Widget for the satusbar.
+    Shows the file name in the status bar.
+    """
+    def __init__(self):
+        super().__init__()
+        
+        self.setObjectName('file_name_status')
+        
+        layout = QtWidgets.QHBoxLayout()
+        self.txt = QtWidgets.QLabel('<< UNSAVED FILE >>')
+
+        # layout.addSpacerItem(QtWidgets.QSpacerItem(500, 5, hData=QtWidgets.QSizePolicy.Maximum))
+        # layout.addStretch()
+        # layout.addSpacerItem(QtWidgets.QSpacerItem(5, 5, hData=QtWidgets.QSizePolicy.Expanding))
+        layout.addWidget(self.txt)
+        
+        self.setLayout(layout)
+
+    def remove_name(self):
+        """Remove the file name in the status bar."""
+        self.txt.setText('')
+
+    def set_name(self, name: str) -> None:
+        """Set the file name in the satus bar."""
+        self.txt.setText(name)
+
+
+
+class PropertyChangedCmd(QtGui.QUndoCommand):
     """
     MODIFIED VERSION.
     
     Node property changed command.
 
     Args:
-        node (NodeGraphQt.NodeObject): node.
+        node (NodeGraphQt6.NodeObject): node.
         name (str): node property name.
         value (object): node property value.
         graph (ElectricalGraph): graph reference.
     """
 
     def __init__(self, node, name, value):
-        QtWidgets.QUndoCommand.__init__(self)
+        QtGui.QUndoCommand.__init__(self)
         
         if name == 'name':
             self.setText('renamed "{}" to "{}"'.format(node.name(), value))
@@ -507,6 +614,21 @@ class PropertyChangedCmd(QtWidgets.QUndoCommand):
             if not storage_row.empty:
                 storage_index = storage_row.index[0]
                 self.graph.net.storage.loc[storage_index, 'in_service'] ^= True
+        elif self.node.type_=='SVCNode.SVCNode' and self.name=='disabled':
+            svc_row = self.graph.net.svc[self.graph.net.svc['name']==self.node.name()]
+            if not svc_row.empty:
+                svc_index = svc_row.index[0]
+                self.graph.net.svc.loc[svc_index, 'in_service'] ^= True
+        elif self.node.type_=='SSCNode.SSCNode' and self.name=='disabled':
+            ssc_row = self.graph.net.ssc[self.graph.net.ssc['name']==self.node.name()]
+            if not ssc_row.empty:
+                ssc_index = ssc_row.index[0]
+                self.graph.net.ssc.loc[ssc_index, 'in_service'] ^= True
+        elif self.node.type_=='TCSCNode.TCSCNode' and self.name=='disabled':
+            tcsc_row = self.graph.net.tcsc[self.graph.net.tcsc['name']==self.node.name()]
+            if not tcsc_row.empty:
+                tcsc_index = tcsc_row.index[0]
+                self.graph.net.tcsc.loc[tcsc_index, 'in_service'] ^= True
 
 
 def return_config(app_root_dir):
@@ -518,14 +640,14 @@ def return_config(app_root_dir):
     the config file exists, but it doesn't have the proper sections
     and options.
     
-    Rertuns the confif file path as a second argument.
+    Rertuns the config file path as a second argument.
     
-    * app_root_dir: Root directory of the applications
+    * app_root_dir: Root directory of the application
     
     Returns:
     
     - First output: the configparser
-    - Second aoutput: the config file path
+    - Second output: the config file path
     """
     config_dir = user_config_dir('electricalsim')
     config_file_path = os.path.join(config_dir, 'config.ini')
@@ -556,3 +678,103 @@ def return_config(app_root_dir):
                     break
                 
     return config, config_file_path
+
+
+def four_ports_on_buses(node):
+    """
+    Change port positions in a bus node.
+    """
+    current_width = node.get_property('width')
+    current_height = node.get_property('height')
+
+    con_outs = node.connected_output_nodes()
+    output_ports = list(con_outs.keys())
+
+    if output_ports[0].view.pos().x() != output_ports[1].view.pos().x():
+        return
+    output_port = output_ports[0]
+    pos = output_port.view.pos()
+    current_x = pos.x()
+    current_y = pos.y()
+    pos.setY(current_y + current_height/2)
+    pos.setX(current_x - current_width/2)
+    output_port.view.setPos(pos)
+
+    output_port2 = output_ports[1]
+    pos = output_port2.view.pos()
+    current_y = pos.y()
+    pos.setY(current_y - current_height/4 - PortEnum.SIZE.value/1.8/2)
+    output_port2.view.setPos(pos)
+
+    con_ins = node.connected_input_nodes()
+    input_ports = list(con_ins.keys())
+
+    input_port = input_ports[0]
+    pos = input_port.view.pos()
+    current_x = pos.x()
+    current_y = pos.y()
+    pos.setX(current_x + current_width/2)
+    pos.setY(current_y - current_height/4 - current_height/3)
+    input_port.view.setPos(pos)
+
+    input_port2 = input_ports[1]
+    pos = input_port2.view.pos()
+    current_y = pos.y()
+    pos.setY(current_y - current_height/4 - PortEnum.SIZE.value/1.8/2)
+    input_port2.view.setPos(pos)
+
+
+def check_new_version():
+    """
+    Checks for available program updates. Returns a first argument True if so,
+    and False otherwise. The second argument is the latest version available.
+    The first output argument is None if there is a connection problem.
+    """
+    url = 'https://pypi.org/pypi/electricalsim/json'
+    try:
+        response = requests.get(url)
+        if response.status_code == 200:
+            latest_version = response.json()["info"]["version"]
+            if latest_version != VERSION:
+                return True, latest_version
+            else:
+                return False, latest_version
+        else:
+            return None, VERSION
+    except requests.exceptions.ConnectionError:
+        return None, VERSION
+    
+
+def open_egs_website():
+    """
+    Open the EGS website in the browser.
+    """
+    QtGui.QDesktopServices.openUrl('https://github.com/aloytag/electrical-grid-simulator')
+
+
+def open_pandapower_documentation():
+    """
+    Open the Pandapower documentation website in the browser.
+    """
+    QtGui.QDesktopServices.openUrl('https://pandapower.readthedocs.io/en/stable/')
+
+
+def open_shortcuts():
+    """
+    Open the list of shortcuts in the EGS website.
+    """
+    QtGui.QDesktopServices.openUrl('https://github.com/aloytag/electrical-grid-simulator/tree/main?tab=readme-ov-file#shortcuts')
+
+
+def open_power_flow_documentation():
+    """
+    Open the Power Flow documentation in the Pandapower online help.
+    """
+    QtGui.QDesktopServices.openUrl('https://pandapower.readthedocs.io/en/stable/powerflow.html')
+
+
+def open_components_documentation():
+    """
+    Open the Pandapower online documentation for the network components.
+    """
+    QtGui.QDesktopServices.openUrl('https://pandapower.readthedocs.io/en/stable/elements.html')
