@@ -1,13 +1,49 @@
 # -*- coding: utf-8 -*-
 
-from Qt import QtCore, QtGui, QtWidgets
+import os
+from PySide6 import QtCore, QtGui, QtWidgets
 
 import numpy as np
 
-from NodeGraphQt import BaseNode, BaseNodeCircle
-from NodeGraphQt.widgets.node_widgets import NodeBaseWidget
+from NodeGraphQt6 import BaseNode
+from NodeGraphQt6.widgets.node_widgets import NodeBaseWidget
 
-from lib.auxiliary import PropertyChangedCmd
+from lib.auxiliary import PropertyChangedCmd, root_dir
+
+icons_dir = os.path.join(root_dir, 'icons')
+
+
+class ImageWrapper(NodeBaseWidget):
+    """
+    Wrapper that allows an image widget (label with pixmap) to be added in a node object.
+    """
+    def __init__(self, parent=None, widget_type=None):
+        super(ImageWrapper, self).__init__(parent)
+
+        # set the name for node property.
+        self.set_name('image')
+
+        # set the label above the widget.
+        # self.set_label('image')
+
+        # set the custom widget.
+        if widget_type is None:
+            widget_type = QtWidgets.QLabel()
+        self.set_custom_widget(widget_type)
+        
+    def get_value(self):
+        widget = self.get_custom_widget()
+        return widget.styleSheet()
+    
+    # def set_value(self, label):
+    #     widget = self.get_custom_widget()
+    #     widget.setFixedSize(100, 100)
+    #     widget.setStyleSheet(label.styleSheet())
+        
+    def set_value(self, style):
+        widget = self.get_custom_widget()
+        widget.setFixedSize(100, 100)
+        widget.setStyleSheet(style)
 
 
 class QSpinBoxWrapper(NodeBaseWidget):
@@ -142,17 +178,6 @@ class BaseNode2(BaseNode):
         for prop in extras:
             self.create_property(prop, False)
         
-        # layout_vert = self.get_property('layout_vert')
-        # if layout_vert is not None and layout_vert is True:
-        #     print('Vertical!!!')
-        #     self.set_layout_direction(1)
-        # elif layout_vert is not None and layout_vert is False:
-        #     self.set_layout_direction(0)
-        #     self.model.set_property('text_color', (255, 255, 255, 180))  # default color
-        # elif layout_vert is None:
-        #     print('Recién se crea layout_vert')
-        #     self.create_property('layout_vert', False)
-        
     def set_property(self, name, value, push_undo=True):
         """
         Set the value on the node custom property.
@@ -190,7 +215,7 @@ class BaseNode2(BaseNode):
 
         Args:
             index (int): index of the port.
-            port (NodeGraphQt.Port): port object.
+            port (NodeGraphQt6.Port): port object.
         """
         src_port = self.input(index)
         src_port.connect_to(port, push_undo)
@@ -201,7 +226,7 @@ class BaseNode2(BaseNode):
 
         Args:
             index (int): index of the port.
-            port (NodeGraphQt.Port): port object.
+            port (NodeGraphQt6.Port): port object.
         """
         src_port = self.output(index)
         src_port.connect_to(port, push_undo)
@@ -375,42 +400,60 @@ class BaseNode2(BaseNode):
         """
         pass
     
-
-class BaseNode2Circle(BaseNodeCircle):
-    def __init__(self):
-        super().__init__()
-        
-    def set_property(self, name, value, push_undo=True):
+    def add_image(self, image_name):
         """
-        Set the value on the node custom property.
-
-        Args:
-            name (str): name of the property.
-            value (object): property data (python built in types).
-            push_undo (bool): register the command to the undo stack. (default: True)
-        """
-
-        # prevent signals from causing a infinite loop.
-        if self.get_property(name) == value:
-            return
-
-        if self.graph and name == 'name':
-            value = self.graph.get_unique_name(value)
-            self.NODE_NAME = value
-
-        if self.graph:
-            if push_undo:
-                undo_stack = self.graph.undo_stack()
-                undo_stack.push(PropertyChangedCmd(self, name, value))
-            else:
-                PropertyChangedCmd(self, name, value).redo()
-        else:
-            if hasattr(self.view, name):
-                setattr(self.view, name, value)
-            self.model.set_property(name, value)
+        Add an image to the node.
         
-        self.update()
+        image_name: Image file name from de 'icons' folder.
+        """
+    
+        self.image_widget = ImageWrapper(self.view)
+        self.image_widget.set_name('image')
+        # icon_path = os.path.join(icons_dir, image_name)
+        # style = f"image: url('{icon_path}');"
+        style = "image: url(:/" + image_name + ");"
+        self.image_widget.set_value(style)
+        self.add_custom_widget(self.image_widget, tab=None)
+        model = self.model
+        self.set_model(model)
 
+    @property
+    def single_port_node(self):
+        """
+        Property: True if the node has only one port, False otherwise.
+        """
+        return len(self.input_ports() + self.output_ports())==1
+
+
+# class BusNode(BaseNode2):
+#     __identifier__ = 'BusNode'
+#     NODE_NAME = 'BusNode'
+    
+#     def __init__(self):
+#         super().__init__()
+#         self.input_port = self.add_input(multi_input=True, display_name=False)
+#         self.output_port = self.add_output(display_name=False)
+#         # self.set_color(157, 157, 157)
+#         self.set_color(117, 117, 117)
+        
+#         self.create_property('bus_index', None)
+        
+#     def node_switch_connected(self):
+#         """
+#         Returns a list of SwitchNode connected to the bus.
+#         Returns an empty list if no switch is connected.
+#         """
+#         switches = []
+#         inputs_nodes = self.connected_input_nodes()[self.input_port]
+#         for node in inputs_nodes:
+#             if node.type_=='SwitchNode.SwitchNode':
+#                 switches.append(node)
+#         outputs_nodes = self.connected_output_nodes()[self.output_port]
+#         for node in outputs_nodes:
+#             if node.type_=='SwitchNode.SwitchNode':
+#                 switches.append(node)
+#         return switches
+    
 
 class BusNode(BaseNode2):
     __identifier__ = 'BusNode'
@@ -420,7 +463,10 @@ class BusNode(BaseNode2):
         super().__init__()
         self.input_port = self.add_input(multi_input=True, display_name=False)
         self.output_port = self.add_output(display_name=False)
-        # self.set_color(157, 157, 157)
+
+        self.input_port2 = self.add_input(name='input2', multi_input=True, display_name=False)
+        self.output_port2 = self.add_output(name='output2', display_name=False)
+
         self.set_color(117, 117, 117)
         
         self.create_property('bus_index', None)
@@ -431,11 +477,13 @@ class BusNode(BaseNode2):
         Returns an empty list if no switch is connected.
         """
         switches = []
-        inputs_nodes = self.connected_input_nodes()[self.input_port]
+        in_dict = self.connected_input_nodes()
+        inputs_nodes = in_dict[self.input_port] + in_dict[self.input_port2]
         for node in inputs_nodes:
             if node.type_=='SwitchNode.SwitchNode':
                 switches.append(node)
-        outputs_nodes = self.connected_output_nodes()[self.output_port]
+        out_dict = self.connected_output_nodes()
+        outputs_nodes = out_dict[self.output_port] + out_dict[self.output_port2]
         for node in outputs_nodes:
             if node.type_=='SwitchNode.SwitchNode':
                 switches.append(node)
@@ -608,6 +656,7 @@ class ImpedanceNode(BaseNode2):
             self.create_property(name, None)
             
         self.create_property('impedance_index', None)
+        self.add_image('impedance.svg')
             
     def connected_to_network(self):
         """
@@ -643,6 +692,7 @@ class TrafoNode(BaseNode2):
                 self.create_property(name, None)
                 
         self.create_property('transformer_index', None)
+        self.add_image('transformer.svg')
             
         # add custom widget to node with "node.view" as the parent.
         self.tap_pos_widget = QSpinBoxWrapper(self.view)
@@ -655,6 +705,13 @@ class TrafoNode(BaseNode2):
         
         Updates the 'tap_pos' parameter on the pandapower network too.
         """
+        # tap_max = self.get_property('tap_max')
+        # if tap_max is not None:
+        #     self.tap_pos_widget.get_custom_widget().setMaximum(tap_max)
+        # tap_min = self.get_property('tap_min')
+        # if tap_min is not None:
+        #     self.tap_pos_widget.get_custom_widget().setMinimum(tap_min)
+
         self.set_property('tap_pos', value, push_undo=False)
         
         transformer_index = self.get_property('transformer_index')
@@ -701,12 +758,14 @@ class StdTrafoNode(BaseNode2):
         self.electrical_properties = ('std_type', 'tap_pos',
                                       'max_loading_percent', 'parallel', 'df',
                                       'vk0_percent', 'vkr0_percent', 'mag0_percent',
-                                      'mag0_rx', 'si0_hv_partial', 'xn_ohm')
+                                      'mag0_rx', 'si0_hv_partial', 'xn_ohm',
+                                      'tap_min', 'tap_max')
         for name in self.electrical_properties:
             if name!='tap_pos':
                 self.create_property(name, None)
                 
         self.create_property('transformer_index', None)
+        self.add_image('transformer.svg')
             
         # add custom widget to node with "node.view" as the parent.
         self.tap_pos_widget = QSpinBoxWrapper(self.view)
@@ -780,6 +839,7 @@ class Trafo3wNode(BaseNode2):
                 self.create_property(name, None)
                 
         self.create_property('transformer_index', None)
+        self.add_image('3w-transformer.svg')
             
         # add custom widget to node with "node.view" as the parent.
         self.tap_pos_widget = QSpinBoxWrapper(self.view)
@@ -844,12 +904,14 @@ class StdTrafo3wNode(BaseNode2):
         
         self.electrical_properties = ('std_type', 'tap_pos',
                                       'max_loading_percent',
-                                      'tap_at_star_point')
+                                      'tap_at_star_point',
+                                      'tap_min', 'tap_max')
         for name in self.electrical_properties:
             if name!='tap_pos':
                 self.create_property(name, None)
                 
         self.create_property('transformer_index', None)
+        self.add_image('3w-transformer.svg')
             
         # add custom widget to node with "node.view" as the parent.
         self.tap_pos_widget = QSpinBoxWrapper(self.view)
@@ -928,6 +990,16 @@ class GenNode(BaseNode2):
                 self.create_property(name, None)
                 
         self.create_property('gen_index', None)
+        
+        # self.image_widget = ImageWrapper(self.view)
+        # pixmap = QtGui.QPixmap(os.path.join(icons_dir, 'generator.svg'))
+        # pixmap = pixmap.scaledToWidth(100, QtCore.Qt.SmoothTransformation)
+        # self.image_widget.set_value(pixmap)
+        # self.add_custom_widget(self.image_widget, tab=None)
+        # model = self.model
+        # self.set_model(model)
+        
+        self.add_image('generator.svg')
                 
         # add custom widget to node with "node.view" as the parent.
         self.p_mw_widget = QSpinBoxWrapper(self.view, widget_type=QtWidgets.QDoubleSpinBox())
@@ -936,6 +1008,9 @@ class GenNode(BaseNode2):
         
         self.p_mw_widget.get_custom_widget().valueChanged.connect(self.update_p_mw)
         self.p_mw_widget.get_custom_widget().setDecimals(5)
+        # max_p = self.get_property('max_p_mw')
+        # if max_p is not None:
+        #     self.p_mw_widget.get_custom_widget().setMaximum(max_p)
         self.add_custom_widget(self.p_mw_widget, tab=None)  # Adds the 'p_mw' property.
         
         # add custom widget to node with "node.view" as the parent.
@@ -1015,7 +1090,7 @@ class GenNode(BaseNode2):
             self.input_port = None
             self.output_port = self.add_output(name='', multi_output=False)
             self.output_port.port_deletion_allowed = True
-            self.set_property('fliped', True, push_undo=False)
+            self.set_property('fliped', True, push_undo=False)        
             
     def connected_to_network(self):
         """
@@ -1058,6 +1133,7 @@ class SGenNode(BaseNode2):
                 self.create_property(name, None)
                 
         self.create_property('gen_index', None)
+        self.add_image('generator.svg')
                 
         # add custom widget to node with "node.view" as the parent.
         self.p_mw_widget = QSpinBoxWrapper(self.view, widget_type=QtWidgets.QDoubleSpinBox())
@@ -1188,6 +1264,7 @@ class ASGenNode(BaseNode2):
             self.create_property(name, None)
             
         self.create_property('gen_index', None)
+        self.add_image('generator.svg')
     
     def flip(self):
         """
@@ -1250,6 +1327,7 @@ class ExtGridNode(BaseNode2):
                 self.create_property(name, None)
                 
         self.create_property('grid_index', None)
+        self.add_image('ext_grid.svg')
         
         # add custom widget to node with "node.view" as the parent.
         self.vm_pu_widget = QSpinBoxWrapper(self.view, widget_type=QtWidgets.QDoubleSpinBox())
@@ -1264,7 +1342,7 @@ class ExtGridNode(BaseNode2):
             
     def update_vm_pu(self, value):
         """
-        Updates 'vm_pu' property when changing the generated active power from the node widget.
+        Updates 'vm_pu' property when changing the value from the node widget.
         
         Updates the 'vm_pu' parameter on the pandapower network too.
         """
@@ -1337,6 +1415,25 @@ class LoadNode(BaseNode2):
                 
         self.create_property('load_index', None)
         # self.create_property('layout_vert', False)
+        
+        # self.image_widget = ImageWrapper(self.view)
+        # pixmap = QtGui.QPixmap(os.path.join(icons_dir, 'demand.svg'))
+        # # pixmap = pixmap.scaled(100, 80, QtCore.Qt.KeepAspectRatio)
+        # pixmap = pixmap.scaledToWidth(50, QtCore.Qt.SmoothTransformation)
+        # self.image_widget.set_value(pixmap)
+        # self.add_custom_widget(self.image_widget, tab=None)
+        # model = self.model
+        # self.set_model(model)
+        
+        self.image_widget = ImageWrapper(self.view)
+        self.image_widget.set_name('image')
+        # icon_path = os.path.join(icons_dir, 'demand.svg')
+        # style = f"image: url('{icon_path}')"
+        style = "image: url(:/demand.svg);"
+        self.image_widget.set_value(style)
+        self.add_custom_widget(self.image_widget, tab=None)
+        model = self.model
+        self.set_model(model)
                 
         # add custom widget to node with "node.view" as the parent.
         self.p_mw_widget = QSpinBoxWrapper(self.view, widget_type=QtWidgets.QDoubleSpinBox())
@@ -1479,6 +1576,24 @@ class ALoadNode(BaseNode2):
             
         self.create_property('load_index', None)
         
+        # self.image_widget = ImageWrapper(self.view)
+        # pixmap = QtGui.QPixmap(os.path.join(icons_dir, 'demand.svg'))
+        # # pixmap = pixmap.scaled(100, 80, QtCore.Qt.KeepAspectRatio)
+        # pixmap = pixmap.scaledToWidth(50, QtCore.Qt.SmoothTransformation)
+        # self.image_widget.set_value(pixmap)
+        # self.add_custom_widget(self.image_widget, tab=None)
+        # model = self.model
+        # self.set_model(model)
+        
+        self.image_widget = ImageWrapper(self.view)
+        self.image_widget.set_name('image')
+        icon_path = os.path.join(icons_dir, 'demand.svg')
+        style = f"image: url('{icon_path}')"
+        self.image_widget.set_value(style)
+        self.add_custom_widget(self.image_widget, tab=None)
+        model = self.model
+        self.set_model(model)
+        
     def flip(self):
         """
         Flip the node (change the input ports by an output port, or Vice versa).
@@ -1538,6 +1653,7 @@ class ShuntNode(BaseNode2):
                 self.create_property(name, None)
                 
         self.create_property('shunt_index', None)
+        self.add_image('shunt.svg')
                 
         # add custom widget to node with "node.view" as the parent.
         self.p_mw_widget = QSpinBoxWrapper(self.view, widget_type=QtWidgets.QDoubleSpinBox())
@@ -1673,6 +1789,7 @@ class MotorNode(BaseNode2):
             self.create_property(name, None)
             
         self.create_property('motor_index', None)
+        self.add_image('motor.svg')
     
     def flip(self):
         """
@@ -1732,6 +1849,7 @@ class WardNode(BaseNode2):
             self.create_property(name, None)
             
         self.create_property('ward_index', None)
+        self.add_image('ward.svg')
         
     def flip(self):
         """
@@ -1793,6 +1911,7 @@ class XWardNode(BaseNode2):
             self.create_property(name, None)
             
         self.create_property('ward_index', None)
+        self.add_image('xward.svg')
         
     def flip(self):
         """
@@ -1857,6 +1976,7 @@ class StorageNode(BaseNode2):
             self.create_property(name, None)
             
         self.create_property('storage_index', None)
+        self.add_image('storage.svg')
         
     def flip(self):
         """
@@ -1907,9 +2027,16 @@ class SwitchNode(BaseNode2):
             self.create_property(name, None)
             
         self.create_property('switch_index', None)
+        self.add_image('switch_closed.svg')
         
         self.add_checkbox('closed', '', 'Closed', True)  # Adds 'closed' property
-        self.get_widget('closed').value_changed.connect(self.update_closed)
+        widget_checkbox = self.get_widget('closed')
+        widget_checkbox.value_changed.connect(self.update_closed)
+        size = widget_checkbox.size()
+        size.scale(150, 150, QtCore.Qt.KeepAspectRatio)
+        widget_checkbox.resize(size)
+        widget_checkbox.get_custom_widget().setStyleSheet("QCheckBox::indicator { width: 50px; height: 50px;}")
+        # self.update()
         
     def update_closed(self, value):
         """
@@ -1922,8 +2049,15 @@ class SwitchNode(BaseNode2):
             self.graph.net.switch.loc[index, 'closed'] = new_value
         if new_value:
             self.set_color(0, 0, 255)
+            # icon_path = os.path.join(icons_dir, 'switch_closed.svg')
+            style = "image: url(:/switch_closed.svg);"
         else:
             self.set_color(178, 199, 218)
+            # icon_path = os.path.join(icons_dir, 'switch_opened.svg')
+            style = "image: url(:/switch_opened.svg);"
+            
+        # style = f"image: url('{icon_path}')"
+        self.image_widget.set_value(style)
             
     def connected_to_network(self):
         """
@@ -1990,3 +2124,259 @@ class SwitchNode(BaseNode2):
             if node.type_ in ('Trafo3wNode.Trafo3wNode', 'StdTrafo3wNode.StdTrafo3wNode'):
                 trafos.append(node)
         return trafos
+
+
+class SVCNode(BaseNode2):
+    __identifier__ = 'SVCNode'
+    NODE_NAME = 'SVCNode'
+    
+    def __init__(self):
+        super().__init__()
+        self.input_port = self.add_input(name='')
+        self.input_port.port_deletion_allowed = True
+        self.output_port = None
+        self.set_port_deletion_allowed(True)
+        self.set_color(206, 145, 115)
+        
+        fliped = self.get_property('fliped')
+        if fliped is not None:
+            self.flip()
+        else:
+            self.create_property('fliped', False)
+        
+        self.electrical_properties = ('x_l_ohm', 'x_cvar_ohm', 'set_vm_pu',
+                                      'thyristor_firing_angle_degree',
+                                      'min_angle_degree', 'controllable',
+                                      'max_angle_degree')
+        
+        for name in self.electrical_properties:
+            if name!='set_vm_pu':
+                self.create_property(name, None)
+                
+        self.create_property('svc_index', None)
+        
+        self.image_widget = ImageWrapper(self.view)
+        self.image_widget.set_name('image')
+        # icon_path = os.path.join(icons_dir, 'demand.svg')
+        # style = f"image: url('{icon_path}')"
+        style = "image: url(:/svc.svg);"
+        self.image_widget.set_value(style)
+        self.add_custom_widget(self.image_widget, tab=None)
+        model = self.model
+        self.set_model(model)
+                
+        # add custom widget to node with "node.view" as the parent.
+        self.vm_pu_widget = QSpinBoxWrapper(self.view, widget_type=QtWidgets.QDoubleSpinBox())
+        self.vm_pu_widget.set_name('set_vm_pu')
+        self.vm_pu_widget.set_label('Vm (p.u.)')
+        self.vm_pu_widget.get_custom_widget().valueChanged.connect(self.update_vm_pu)
+        self.vm_pu_widget.get_custom_widget().setDecimals(4)
+        self.vm_pu_widget.get_custom_widget().setMinimum(0.0)
+        self.vm_pu_widget.get_custom_widget().setMaximum(2.0)
+        self.vm_pu_widget.get_custom_widget().setSingleStep(0.01)
+        self.add_custom_widget(self.vm_pu_widget, tab=None)  # Adds the 'vm_pu' property.
+            
+    def update_vm_pu(self, value):
+        """
+        Updates 'set_vm_pu' property when changing the value from the node widget.
+        
+        Updates the 'set_vm_pu' parameter on the pandapower network too.
+        """
+        self.set_property('set_vm_pu', value, push_undo=False)
+        
+        svc_index = self.get_property('svc_index')
+        if svc_index is not None and self.connected_to_network():
+            self.graph.net.svc.loc[svc_index, 'set_vm_pu'] = value
+    
+    def flip(self):
+        """
+        Flip the node (change the input ports by an output port, or Vice versa).
+        """
+        try:
+            output_port = self.output_ports()[0]
+            output_port.clear_connections(push_undo=False)
+            self.delete_output(0)
+            self.output_port = None
+            self.input_port = self.add_input(name='')
+            self.input_port.port_deletion_allowed = True
+            self.set_property('fliped', False, push_undo=False)
+        except IndexError:
+            input_port = self.input_ports()[0]
+            input_port.clear_connections(push_undo=False)
+            self.delete_input(0)
+            self.input_port = None
+            self.output_port = self.add_output(name='', multi_output=False)
+            self.output_port.port_deletion_allowed = True
+            self.set_property('fliped', True, push_undo=False)
+            
+    def connected_to_network(self):
+        """
+        Returns True if the SVC node is connected to the network (to a bus).
+        Returns False otherwise.
+        """        
+        try:
+            connections = len(self.connected_output_nodes()[self.output_ports()[0]])
+        except IndexError:
+            connections = len(self.connected_input_nodes()[self.input_ports()[0]])
+        return connections
+
+
+class SSCNode(BaseNode2):
+    __identifier__ = 'SSCNode'
+    NODE_NAME = 'SSCNode'
+    
+    def __init__(self):
+        super().__init__()
+        self.input_port = self.add_input(name='')
+        self.input_port.port_deletion_allowed = True
+        self.output_port = None
+        self.set_port_deletion_allowed(True)
+        self.set_color(206, 145, 115)
+        
+        fliped = self.get_property('fliped')
+        if fliped is not None:
+            self.flip()
+        else:
+            self.create_property('fliped', False)
+        
+        self.electrical_properties = ('r_ohm', 'x_ohm', 'set_vm_pu',
+                                      'vm_internal_pu',
+                                      'va_internal_degree',
+                                      'controllable')
+        
+        for name in self.electrical_properties:
+            if name!='set_vm_pu':
+                self.create_property(name, None)
+                
+        self.create_property('ssc_index', None)
+        
+        self.image_widget = ImageWrapper(self.view)
+        self.image_widget.set_name('image')
+        # icon_path = os.path.join(icons_dir, 'demand.svg')
+        # style = f"image: url('{icon_path}')"
+        style = "image: url(:/ssc.svg);"
+        self.image_widget.set_value(style)
+        self.add_custom_widget(self.image_widget, tab=None)
+        model = self.model
+        self.set_model(model)
+                
+        # add custom widget to node with "node.view" as the parent.
+        self.vm_pu_widget = QSpinBoxWrapper(self.view, widget_type=QtWidgets.QDoubleSpinBox())
+        self.vm_pu_widget.set_name('set_vm_pu')
+        self.vm_pu_widget.set_label('Vm (p.u.)')
+        self.vm_pu_widget.get_custom_widget().valueChanged.connect(self.update_vm_pu)
+        self.vm_pu_widget.get_custom_widget().setDecimals(4)
+        self.vm_pu_widget.get_custom_widget().setMinimum(0.0)
+        self.vm_pu_widget.get_custom_widget().setMaximum(2.0)
+        self.vm_pu_widget.get_custom_widget().setSingleStep(0.01)
+        self.add_custom_widget(self.vm_pu_widget, tab=None)  # Adds the 'vm_pu' property.
+            
+    def update_vm_pu(self, value):
+        """
+        Updates 'set_vm_pu' property when changing the value from the node widget.
+        
+        Updates the 'set_vm_pu' parameter on the pandapower network too.
+        """
+        self.set_property('set_vm_pu', value, push_undo=False)
+        
+        ssc_index = self.get_property('ssc_index')
+        if ssc_index is not None and self.connected_to_network():
+            self.graph.net.ssc.loc[ssc_index, 'set_vm_pu'] = value
+    
+    def flip(self):
+        """
+        Flip the node (change the input ports by an output port, or Vice versa).
+        """
+        try:
+            output_port = self.output_ports()[0]
+            output_port.clear_connections(push_undo=False)
+            self.delete_output(0)
+            self.output_port = None
+            self.input_port = self.add_input(name='')
+            self.input_port.port_deletion_allowed = True
+            self.set_property('fliped', False, push_undo=False)
+        except IndexError:
+            input_port = self.input_ports()[0]
+            input_port.clear_connections(push_undo=False)
+            self.delete_input(0)
+            self.input_port = None
+            self.output_port = self.add_output(name='', multi_output=False)
+            self.output_port.port_deletion_allowed = True
+            self.set_property('fliped', True, push_undo=False)
+            
+    def connected_to_network(self):
+        """
+        Returns True if the SVC node is connected to the network (to a bus).
+        Returns False otherwise.
+        """        
+        try:
+            connections = len(self.connected_output_nodes()[self.output_ports()[0]])
+        except IndexError:
+            connections = len(self.connected_input_nodes()[self.input_ports()[0]])
+        return connections
+
+
+class TCSCNode(BaseNode2):
+    __identifier__ = 'TCSCNode'
+    NODE_NAME = 'TCSCNode'
+    
+    def __init__(self):
+        super().__init__()
+        self.input_port = self.add_input(name='from')
+        self.output_port = self.add_output(name='to', multi_output=False)
+        self.set_color(206, 145, 115)
+        
+        self.electrical_properties = ('x_l_ohm', 'x_cvar_ohm', 'set_p_to_mw',
+                                      'thyristor_firing_angle_degree',
+                                      'min_angle_degree', 'controllable',
+                                      'max_angle_degree')
+        
+        for name in self.electrical_properties:
+            if name!='set_p_to_mw':
+                self.create_property(name, None)
+                
+        self.create_property('tcsc_index', None)
+        
+        self.image_widget = ImageWrapper(self.view)
+        self.image_widget.set_name('image')
+        # icon_path = os.path.join(icons_dir, 'tcsc.svg')
+        # style = f"image: url('{icon_path}')"
+        style = "image: url(:/tcsc.svg);"
+        self.image_widget.set_value(style)
+        self.add_custom_widget(self.image_widget, tab=None)
+        model = self.model
+        self.set_model(model)
+            
+        # add custom widget to node with "node.view" as the parent.
+        self.p_mw_widget = QSpinBoxWrapper(self.view, widget_type=QtWidgets.QDoubleSpinBox())
+        self.p_mw_widget.set_name('set_p_to_mw')
+        self.p_mw_widget.set_label('P (MW)')
+        # self.p_mw_widget.set_custom_widget(QtWidgets.QDoubleSpinBox())
+        # self.p_mw_widget.get_custom_widget().valueChanged.connect(self.set_p_to_mw)
+        self.p_mw_widget.get_custom_widget().setDecimals(4)
+        self.p_mw_widget.get_custom_widget().setMinimum(-5000.0)
+        self.p_mw_widget.get_custom_widget().setMaximum(5000.0)
+        self.add_custom_widget(self.p_mw_widget, tab=None)  # Adds the 'set_p_to_mw' property.
+        self.p_mw_widget.get_custom_widget().valueChanged.connect(self.update_p_to_mw)
+            
+    def update_p_to_mw(self, value):
+        """
+        Updates 'set_p_to_mw' property when changing the power transmission widget from the node.
+        
+        Updates the 'set_p_to_mw' parameter on the pandapower network too.
+        """
+        self.set_property('set_p_to_mw', value, push_undo=False)
+        
+        tcsc_index = self.get_property('tcsc_index')
+        if tcsc_index is not None and self.connected_to_network():
+            self.graph.net.tcsc.loc[tcsc_index, 'set_p_to_mw'] = value
+            
+    def connected_to_network(self):
+        """
+        Returns True if the TCSC node is connected to the network (to buses).
+        Returns False otherwise.
+        """
+        inputs_connected = len(self.connected_input_nodes()[self.input_port])
+        outputs_connected = len(self.connected_output_nodes()[self.output_port])
+        
+        return inputs_connected * outputs_connected
